@@ -1,4 +1,4 @@
-let version = "0.3.8.1"
+let version = "0.9.1"
 let conf = Dtools.Conf.void "Liquidsoap configuration"
 let var_script = ref "default"
 let substs = ref
@@ -14,7 +14,6 @@ let subst_vars s =
      s !substs
 let env_has key = try ignore (Sys.getenv key) ; true with Not_found -> false
 let dynliq_option = []
-let resample = None
 (* The (*#*) is here to fake a comment line for configure.
  * If not, then the [] may be stripped... *)
 let magic_cookie = Magic.create (*#*) ~flags:[Magic.Mime] []
@@ -41,9 +40,9 @@ let get_encoding () =
     | Some e -> e
     | None ->
         let encs = conf_encoding#get in
-	let e = C.automatic "LIQ-TAGS" (List.map enc_of_name encs) C.utf8 in
-        custom_encoding := Some e ;
-	e
+        let e = C.automatic "LIQ-TAGS" (List.map enc_of_name encs) C.utf8 in
+          custom_encoding := Some e ;
+          e
 
 let recode_tag ?encoding s =
   let in_enc =
@@ -63,7 +62,7 @@ let requests_table_size = 50
 let tts_program = "/usr/local/lib/liquidsoap/liquidtts"
 let rundir = "/usr/local/var/run/liquidsoap"
 let logdir = "/usr/local/var/log/liquidsoap"
-let libs_dir = "/usr/local/lib/liquidsoap/0.3.8.1"
+let libs_dir = "/usr/local/lib/liquidsoap/0.9.1"
 let display_types = ref false
 let () = add_subst "<sysrundir>" "/usr/local/var/run/liquidsoap"
 let () = add_subst "<syslogdir>" "/usr/local/var/log/liquidsoap"
