@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,17 +33,39 @@ let conf_samplerate =
   Conf.int ~p:(conf#plug "samplerate") ~d:44100
     "Internal samplerate"
 let conf_size =
-  Conf.int ~p:(conf#plug "size") ~d:1024
+  Conf.int ~p:(conf#plug "size") ~d:1764
     "Frame size in samples"
     ~comments:[
       "Tweaking this is tricky but can be useful, since the latency is" ;
       "proportional to the frame size. It can also help to get ALSA correctly" ;
-      "synchronized with liquidsoap when used in non-buffered mode."
+      "synchronized with liquidsoap when used in non-buffered mode." ;
+      "With video enabled, frame duration should be a multiple of ";
+      "audio and video frame duration, in order to maintain synchronization."
     ]
 let conf_channels =
   Conf.int ~p:(conf#plug "channels") ~d:2
     "Number of channels"
 
+let conf_video =
+  Conf.void ~p:(conf#plug "video") "Video format"
+      ~comments:[
+        "Parameters for video in liquidsoap."
+      ]
+
+(* TODO: better organisation.. *)
+let conf_video_channels =
+  Conf.int ~p:(conf_video#plug "channels") ~d:0
+    "Video channels"
+let conf_video_width = 
+  Conf.int ~p:(conf_video#plug "width") ~d:320
+    "Video frame width"
+let conf_video_height =
+  Conf.int ~p:(conf_video#plug "height") ~d:240
+    "Video frame height"
+let conf_video_fps =
+  Conf.int ~p:(conf_video#plug "fps") ~d:25
+    "Video frames per second"
+
 (* Global time speed. *)
 let samples_per_second () = conf_samplerate#get
 
@@ -54,14 +76,6 @@ let samples_per_frame () = conf_size#get
 (** Number of channels. *)
 let channels () = conf_channels#get
 
-(* Create a standard audio frame. *)
-let create_frame () =
-  Frame.create
-    (Array.init (channels())
-       (fun _ ->
-          Frame.Float_pcm_t (float (samples_per_second()))))
-    (samples_per_second()) (samples_per_frame())
-
 (* Some day the time reference might not be the standard audio sample. *)
 let ticks_per_second = samples_per_second
 let ticks_per_frame = samples_per_frame
@@ -76,3 +90,15 @@ let ticks_of_samples s = s * (ticks_per_second() / samples_per_second())
 let samples_of_ticks t = t / (ticks_per_second() / samples_per_second())
 let seconds_of_samples i = float i /. float (samples_per_second ())
 let seconds_of_ticks i   = float i /. float (ticks_per_second ())
+
+let video_frames_per_second () = conf_video_fps#get
+
+let video_frames_per_frame () = int_of_float (float (video_frames_per_second ()) *. (seconds_per_frame ()))
+let video_frames_of_seconds s = int_of_float (float (video_frames_per_second ()) *. s)
+let ticks_of_video_frames f = (ticks_per_second ()) * f / (video_frames_per_second ())
+let video_frames_of_ticks t = t / (ticks_per_second () / video_frames_per_second ())
+
+let video_width () = conf_video_width#get
+let video_height () = conf_video_height#get
+
+let video_channels () = conf_video_channels#get
