@@ -17,11 +17,6 @@
 
 open Dtools
 
-val conf : Conf.ut
-val conf_samplerate : int Conf.t
-val conf_size : int Conf.t
-val conf_channels : int Conf.t
-
 (** Number of samples in a second. *)
 val samples_per_second : unit -> int
 
@@ -31,9 +26,6 @@ val samples_per_frame : unit -> int
 (** Number of audio channels in a frame. *)
 val channels : unit -> int
 
-(** Helper function to create an audio frame with the right parameters. *)
-val create_frame : unit -> Frame.t
-
 (** Number of ticks in a second. *)
 val ticks_per_second : unit -> int
 
@@ -63,3 +55,20 @@ val ticks_of_samples : int -> int
 
 (** Get the number of samples in a given number of ticks. *)
 val samples_of_ticks : int -> int
+
+
+
+val video_channels : unit -> int
+
+val video_frames_per_frame : unit -> int
+
+val video_frames_per_second : unit -> int
+
+val video_frames_of_seconds : float -> int
+
+val ticks_of_video_frames : int -> int
+
+val video_frames_of_ticks : int -> int
+
+val video_width : unit -> int
+val video_height : unit -> int
