@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -107,6 +107,7 @@ object
   method remaining = -1
   method abort_track  = ()
   method output_reset = ()
+  method is_active = true
 
   val mutable ring = [||]
   initializer
@@ -121,6 +122,17 @@ object
 
   val mutable port = [||]
 
+  val sample_converters =
+    Array.init (Array.length port_names)
+      (fun _ -> Audio_converter.Samplerate.create 1)
+
+  method resample i ratio b ofs len =
+   let conv = sample_converters.(i) in
+   let ret =
+     Audio_converter.Samplerate.resample conv ratio [|b|] ofs len
+   in
+   ret.(0)
+
   method output_get_ready =
     let c = client () in
       port <-
@@ -212,7 +224,7 @@ object (self)
         in
         (* Resample data. *)
         let buf =
-          if coef = 1. then buf else Float_pcm.Channel.resample coef buf 0 n
+          if coef = 1. then buf else self#resample chan coef buf 0 n
         in
         let buflen = Array.length buf in
         let chanlen = Array.length dest.(chan) in
@@ -259,7 +271,7 @@ object (self)
 
       for chan = 0 to Array.length port_names - 1 do
         let buf =
-          Float_pcm.Channel.resample
+          self#resample chan 
             (float (Jack.Client.get_sample_rate (client ())) /. float (Fmt.samples_per_second ()))
             s.(chan)
             0
