@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -76,13 +76,14 @@ let add_builtin ~category ~descr ?(flags=[]) name proto return_t f =
 (** Specialized version for operators, that is builtins returning sources. *)
 
 type category =
-  | Input | Output | TrackProcessing | SoundProcessing | Visualization
+  | Input | Output | TrackProcessing | SoundProcessing | VideoProcessing | Visualization
 
 let string_of_category x = "Source / " ^ match x with
   | Input -> "Input"
   | Output -> "Output"
   | TrackProcessing -> "Track Processing"
   | SoundProcessing -> "Sound Processing"
+  | VideoProcessing -> "Video Processing"
   | Visualization -> "Visualization"
 
 let add_operator ~category ~descr ?(flags=[]) name proto f =
@@ -165,6 +166,13 @@ let to_string t = match t.value with
   | String s -> s
   | _ -> assert false
 
+let to_string_getter t = match t.value with
+  | String s -> (fun () -> s)
+  | Fun _ | FFI _ ->
+      (fun () ->
+         match (apply t []).value with String s -> s | _ -> assert false)
+  | _ -> assert false
+
 let to_float t = match t.value with
   | Float s -> s
   | _ -> assert false
@@ -245,6 +253,7 @@ let product_t a b = T.make (T.Product (a,b))
 let fun_t p b = T.make (T.Arrow (p,b))
 let univ_t ?(constraints=[]) i = T.make (T.UVar (i,constraints))
 let metadata_t = list_t (product_t string_t string_t)
+let string_getter_t n = univ_t ~constraints:[T.Getter T.String] n
 let float_getter_t n = univ_t ~constraints:[T.Getter T.Float] n
 
 let mk v = { t = T.dummy ; value = v }
