@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -84,6 +84,7 @@ type category =
   | Output (** Output. *)
   | TrackProcessing (** Operations on tracks (e.g. mixing, etc.). *)
   | SoundProcessing (** Operations on sound (e.g. compression, etc.). *)
+  | VideoProcessing (** Operations on video. *)
   | Visualization (** Visializations of the sound. *)
 
 (** Get a string representation of a [doc_flag]. *)
@@ -102,6 +103,7 @@ val add_operator :
 
 val to_bool : value -> bool
 val to_string : value -> string
+val to_string_getter : value -> unit -> string
 val to_float : value -> float
 val to_float_getter : value -> unit -> float
 val to_source : value -> Source.source
@@ -143,6 +145,9 @@ val univ_t     : ?constraints:Lang_types.constraints -> int -> kind
 (** A shortcut for lists of pairs of strings. *)
 val metadata_t : kind
 
+(** A string getter. The argument is the number of the universal type parameter
+  * (should be >= 1). *)
+val string_getter_t : int -> kind
 (** A float getter. The argument is the number of the universal type parameter
   * (should be >= 1). *)
 val float_getter_t : int -> kind
