@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -137,6 +137,63 @@ let () =
       Tutils.shutdown () ;
       Lang.unit)
 
+let () =
+  let process_t =
+    Lang.fun_t
+      [false,"",Lang.string_t] Lang.string_t
+  in
+    add_builtin "add_decoder" ~cat:Liq ~descr:"Register an external file decoder. \
+                                           The encoder should output in WAV format \
+                                           to his standard output (stdout)."
+      ["",Lang.string_t,None,Some "Format/decoder's name." ;
+       "",process_t,None,Some "Process to start. The function \
+                           takes the filename as argument and \
+                           returns the process to start."  ]
+      Lang.unit_t
+      (fun p ->
+         let name = Lang.to_string (Lang.assoc "" 1 p) in
+         let f = Lang.assoc "" 2 p in
+         let decoder name = 
+           Lang.to_string (Lang.apply f ["",Lang.string name])
+         in
+         Externalformat.register_external_decoder name decoder;
+         Lang.unit)
+
+let () =
+  let resolver_t =
+    Lang.fun_t
+      [false,"format",Lang.string_t;
+       false,"",Lang.string_t] 
+         (Lang.list_t (Lang.product_t 
+                        Lang.string_t Lang.string_t))
+  in
+    add_builtin "add_metadata_resolver" 
+          ~cat:Liq ~descr:"Register an external file metadata decoder."
+      ["",Lang.string_t,None,Some "Format/resolver's name." ;
+       "",resolver_t,None,Some "Process to start. The function \
+                           takes the format and filename as argument \
+                           and returns a list of (name,value) \
+                           fields."  ]
+      Lang.unit_t
+      (fun p ->
+         let format = Lang.to_string (Lang.assoc "" 1 p) in
+         let f = Lang.assoc "" 2 p in
+         let resolver ~format name =
+           let ret = Lang.apply f ["format",Lang.string format;
+                                   "",Lang.string name] 
+           in
+           let ret = Lang.to_list ret in
+           let ret = List.map Lang.to_product ret in
+           let ret = 
+             List.map 
+               (fun (x,y) -> Lang.to_string x,Lang.to_string y) 
+               ret
+           in
+           ret
+         in
+         Externalformat.register_external_metadata_resolver format resolver;
+         Lang.unit)
+
 let () =
   let protocol_t =
     Lang.fun_t
@@ -144,12 +201,15 @@ let () =
       (Lang.list_t Lang.string_t)
   in
     add_builtin "add_protocol" ~cat:Liq ~descr:"Register a new protocol."
-      ["",Lang.string_t,None,None ;
+      ["temporary",Lang.bool_t,Some (Lang.bool false),
+       Some "if true, file removed when it is finished.";
+       "",Lang.string_t,None,None ;
        "",protocol_t,None,None ]
       Lang.unit_t
       (fun p ->
          let name = Lang.to_string (Lang.assoc "" 1 p) in
          let f = Lang.assoc "" 2 p in
+         let temporary = Lang.to_bool (List.assoc "temporary" p) in
            Request.protocols#register name
              { Request.static = false ;
                Request.resolve =
@@ -160,7 +220,7 @@ let () =
                    in
                      List.map
                        (fun s ->
-                          Request.indicator (Lang.to_string s))
+                          Request.indicator ~temporary (Lang.to_string s))
                        (Lang.to_list l) } ;
            Lang.unit)
 
@@ -189,6 +249,15 @@ let () =
 
 (** Math *)
 
+let () =
+  let t = Lang.univ_t ~constraints:[Lang_types.Num] 1 in
+    add_builtin "~-" ~cat:Math ~descr:"Returns the opposite of its argument."
+      ["",t,None,None] t
+      (function
+         | ["",{ Lang.value = Lang.Int i }] -> Lang.int (~- i)
+         | ["",{ Lang.value = Lang.Float i }] -> Lang.float (~-. i)
+         | _ -> assert false)
+
 let () =
   let t = Lang.univ_t ~constraints:[Lang_types.Num] 1 in
     add_builtin "abs" ~cat:Math ~descr:"Absolute value."
@@ -433,8 +502,8 @@ let () =
 
 let () =
   add_builtin "string.capitalize" ~cat:String
-    ~descr:"Return a string with the first character set to lower case (capitalize), \
-            or to lower case (uncapitalize)."
+    ~descr:"Return a string with the first character set to upper case \
+            (capitalize), or to lower case (uncapitalize)."
     [ "capitalize", Lang.bool_t, Some (Lang.bool true), 
         Some "Capitalize if true, uncapitalize otherwise";
       "space_sensitive", Lang.bool_t, Some (Lang.bool true),
@@ -559,7 +628,7 @@ let () =
     (fun p ->
        let f,l =
          match p with
-           | (_,f)::(_,l)::_ -> f,l
+           | [("",f);("",l)] -> f,l
            | _ -> assert false
        in
        let l = Lang.to_list l in
@@ -599,7 +668,8 @@ let () =
 
 let () =
   add_builtin "list.hd" ~cat:List
-    ~descr:"Return the head of a list, i.e. its first element."
+    ~descr:"Return the head (first element) of a list, \
+            or \"\" if the list is empty."
     ["",Lang.list_t Lang.string_t,None,None] Lang.string_t
     (fun p ->
        try
@@ -933,8 +1003,9 @@ let () =
     ~descr:"Register a command. You can then execute this function \
             through the server, either telnet or socket."
     [ "namespace",Lang.string_t,Some (Lang.string ""),None ;
-      "description",Lang.string_t,Some (Lang.string "No documentation available."),
-      None ;
+      "description",Lang.string_t,
+      Some (Lang.string "No documentation available."),
+      Some "A description of your command." ;
       "usage",Lang.string_t,Some (Lang.string ""),None ;
       "",Lang.string_t,None,None ;
       "",Lang.fun_t [false,"",Lang.string_t] Lang.string_t,None,None ]
@@ -946,9 +1017,7 @@ let () =
        let command = Lang.to_string (Lang.assoc "" 1 p) in
        let f = Lang.assoc "" 2 p in
        let f x = Lang.to_string (Lang.apply f ["",Lang.string x]) in
-       let ns =
-         Server.register (Pcre.split ~pat:"\\." namespace) "custom commands"
-       in
+       let ns = (Pcre.split ~pat:"\\." namespace) in
        let usage = if usage = "" then command ^ " <variable>" else usage in
            Server.add ~ns ~usage ~descr command f ;
          Lang.unit)
@@ -1212,7 +1281,7 @@ struct
     let usage = "set <variable> = <value>" in
       Server.add ~ns ~usage "set" ~descr:"Set a variable's value."
         (fun s ->
-           let pat = "^([a-zA-Z_][a-zA-Z0-9_.]*) *= *([^ ]+)" in
+           let pat = "^([a-zA-Z_][a-zA-Z0-9_.]*) *= *(\"[^\"]*\"|[^ ]+)" in
            try
              let sub = Pcre.exec ~pat s in
              let name = Pcre.get_substring sub 1 in
@@ -1256,7 +1325,29 @@ struct
 end
 
 let () =
-  add_builtin "interactive_float" ~cat:Interaction
+  add_builtin "interactive.string" ~cat:Interaction
+    ~descr:"Read a string from an interactive input."
+    ["",Lang.string_t,None,None; "",Lang.string_t,None,None ]
+    (Lang.fun_t [] Lang.string_t)
+    (fun p ->
+       let name = Lang.to_string (Lang.assoc "" 1 p) in
+       let v = Lang.to_string (Lang.assoc "" 2 p) in
+       let v = ref v in
+         Var.add
+           name
+           Lang.string_t
+           ~get:(fun () -> Printf.sprintf "%S" !v)
+           ~set:(fun s -> v := (Scanf.sscanf s "%S" (fun s -> s)))
+           ~validate:(fun s ->
+                        try
+                          ignore (Scanf.sscanf s "%S" (fun s -> s))
+                        with _ ->
+                          raise (Var.Invalid_value
+                                   (s ^ " is not a string")));
+         Lang.val_fun [] (fun p -> Lang.string !v))
+
+let () =
+  add_builtin "interactive.float" ~cat:Interaction
     ~descr:"Read a float from an interactive input."
     ["",Lang.string_t,None,None; "",Lang.float_t,None,None ]
     (Lang.fun_t [] Lang.float_t)
