@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,9 @@ object (self)
 
   val mutable data = Array.make (Fmt.channels()) [||]
 
-  val databuf = Fmt.create_frame ()
+  val databuf = Frame.make ()
+
+  val converter = Audio_converter.Samplerate.create (Fmt.channels ())
 
   val mutable dpos = 0
 
@@ -51,7 +53,8 @@ object (self)
             AFrame.clear databuf;
             source#get databuf;
             let db = AFrame.get_float_pcm databuf in
-              data <- Float_pcm.resample (ratio ()) db 0 (Array.length db.(0));
+              data <- Audio_converter.Samplerate.resample converter
+                         (ratio ()) db 0 (Array.length db.(0));
               dpos <- 0
           );
         for c = 0 to (Fmt.channels()) - 1 do
