@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ object (self)
   method abort_track = source#abort_track
 
   (* Temporary buffer. *)
-  val databuf = Fmt.create_frame ()
+  val databuf = Frame.make ()
 
   method get_frame buf =
     let b = AFrame.get_float_pcm buf in
