@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ object (self)
 
   inherit operator (List.map (fun x -> x.source) cases) as super
 
-  val mutable selected = None
+  val mutable selected : (child*source) option = None
 
   (** We have to explictly manage our children as they are dynamically created
     * by application of the transition functions. In particular we need a list
@@ -81,10 +81,19 @@ object (self)
             c
 
   method after_output =
+    (* Advance the memo frame. *)
     self#advance ;
+    (* Propagate to all sub-sources, i.e. our children and the transitions
+     * wrapping them:
+     *  - current transition is [selected],
+     *  - old one in [to_finish]. *)
+    List.iter (fun s -> s.source#after_output) cases ;
+    begin match selected with
+      | None -> ()
+      | Some (_,s) -> s#after_output
+    end ;
     List.iter (fun s -> s#after_output) to_finish ;
     to_finish <- [] ;
-    List.iter (fun s -> s.source#after_output) cases ;
     (* Selection may have been triggered by a call to #is_ready, without
      * any call to #get_ready (in particular if #select returned None).
      * It is cleared here in order to get a chance to be re-computed later. *)
@@ -108,7 +117,10 @@ object (self)
          s#leave ~dynamic:true (self:>source) ;
          Lang.iter_sources (fun s -> s#leave ~dynamic:true (self:>source))
            transition)
-      cases
+      cases ;
+    match selected with
+      | None -> ()
+      | Some (_,s) -> s#leave (self:>source)
 
   method is_ready = need_eot || selected <> None || self#cached_select <> None
 
@@ -123,14 +135,14 @@ object (self)
       | Some c ->
           begin match selected with
             | None ->
-                self#log#f 3 "switch to %s" c.source#id ;
+                self#log#f 3 "Switch to %s." c.source#id ;
                 (* The source is already ready, this call is only there for
                  * allowing an uniform treatment of switches, triggering
                  * a #leave call. *)
                 c.source#get_ready activation ;
                 selected <- Some (c,c.source)
             | Some (old_c,old_s) when old_c != c ->
-                self#log#f 3 "switch to %s with%s transition"
+                self#log#f 3 "Switch to %s with%s transition."
                   c.source#id
                   (if forget then " forgetful" else "") ;
                 old_s#leave (self:>source) ;
