@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,18 +28,13 @@ let no_mount = "Use [name]"
 let no_name = "Use [mount]"
 
 let proto =
-  Icecast2.proto @
-  [ "mount", Lang.string_t, Some (Lang.string no_mount), None ;
-    "name", Lang.string_t, Some (Lang.string no_name), None ;
-
-    "samplerate", Lang.int_t, Some (Lang.int 44100), None;
+  (Icecast2.proto ~no_mount ~no_name) @
+  [ "samplerate", Lang.int_t, Some (Lang.int 44100), None;
     "bitrate", Lang.int_t, Some (Lang.int 128), None;
     "quality", Lang.int_t, Some (Lang.int 5), None;
     "stereo", Lang.bool_t, Some (Lang.bool true), None;
-    "protocol", Lang.string_t, (Some (Lang.string "http")),
-    Some "Protocol of the streaming server: \
-          'http' for Icecast, 'icy' for Shoutcast." ;
-
+    "start", Lang.bool_t, Some (Lang.bool true),
+    Some "Start output threads on operator initialization." ;
     "", Lang.source_t, None, None ]
 
 let no_multicast = "no_multicast"
@@ -54,7 +49,7 @@ class to_shout p =
   let quality = e Lang.to_int "quality" in
 
   let source = List.assoc "" p in
-
+  let autostart = Lang.to_bool (List.assoc "start" p) in
   let mount = s "mount" in
   let name = s "name" in
   let name =
@@ -78,16 +73,29 @@ class to_shout p =
         | "icy" -> Shout.Protocol_icy
         | _ ->
             raise (Lang.Invalid_value
-                     (v, "valid values are 'http' (icecast) "^
-                      "and 'icy' (shoutcast)"))
+                     (v, "valid values are 'http' (icecast) \
+                          and 'icy' (shoutcast)"))
+  in
+  let channels =
+    if not stereo then 1 else Fmt.channels ()
+  in
+  let icecast_info =
+    {
+     Icecast2.
+      quality    = None;
+      bitrate    = Some bitrate;
+      channels   = Some channels;
+      samplerate = Some samplerate
+    }
   in
 object (self)
+  inherit Output.encoded ~autostart ~name:mount ~kind:"output.icecast" source
   inherit
-    [Lame.encoder] Icecast2.output ~format:Shout.Format_mp3 ~protocol
-      ~bitrate:(string_of_int bitrate) ~mount ~name ~source p as super
-  inherit base
+    Icecast2.output ~format:Shout.Format_mp3 ~protocol
+      ~icecast_info ~mount ~name ~source p as icecast
+  inherit base ~quality ~bitrate ~stereo ~samplerate as base
 
-  method reset_encoder encoder m =
+  method reset_encoder m =
     let get h k l =
       try
         (k,(Hashtbl.find h k))::l
@@ -136,17 +144,20 @@ object (self)
         | None -> ""
 
   method output_start =
-    super#output_start ;
-    self#log#f 3 "Setting up an MP3 encoder..." ;
-    let enc = create_encoder ~bitrate ~samplerate ~quality ~stereo in
-      encoder <- Some enc
+    icecast#icecast_start ;
+    base#output_start 
+
+  method output_stop = icecast#icecast_stop
+
+  method output_reset = 
+    self#output_stop;
+    self#output_start
 
 end
 
 let () =
-    Lang.add_operator "output.icecast.mp3" ~category:Lang.Output
-      ~descr:
-  "Output the source's stream to an icecast2 compatible server in MP3 format."
-      proto
-      (fun p -> ((new to_shout p):>Source.source))
-
+  Lang.add_operator "output.icecast.mp3" ~category:Lang.Output
+    ~descr:"Output the source's stream to an icecast2-compatible server \
+            in MP3 format."
+    proto
+    (fun p -> ((new to_shout p):>Source.source))
