@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ object
 
   val mutable start_output : bool
   val mutable stop_output : bool
+  method is_active : bool
 
   method add_metadata : Request.metadata -> unit
   method metadata_queue : Request.metadata Queue.t
@@ -48,7 +49,7 @@ object
   method virtual output_stop : unit
 end
 
-class virtual ['a] encoded :
+class virtual encoded :
   kind:string ->
   name:string ->
   autostart:bool ->
@@ -58,9 +59,8 @@ object
 
   method output_send : Frame.t -> unit
 
-  val mutable encoder : 'a option
-  method virtual encode : 'a -> float array array -> int -> int -> string
-  method virtual reset_encoder : 'a -> AFrame.metadata -> string
+  method virtual encode : Frame.t -> int -> int -> string
+  method virtual reset_encoder : AFrame.metadata -> string
   method virtual send : string -> unit
 
   method virtual output_reset : unit
