@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,10 @@ open Http_source
 
 exception NoDecoder
 
-class http_input_server ~dumpfile ~bufferize ~max ~on_connect ~on_disconnect ~login =
+class http_input_server ~dumpfile ~logfile
+                        ~bufferize ~max
+                        ~on_connect ~on_disconnect
+                        ~login ~debug =
   let abg_max_len = Fmt.samples_of_seconds max in
 object (self)
   inherit Source.source
@@ -36,25 +39,26 @@ object (self)
             (Generator.create ())
             ~empty_on_abort:false ~bufferize
 
-   val mutable relaying = false
-   val mutable ns = []
-   val mutable decoder = fun _ -> raise NoDecoder
-   val mutable stype = None
-   val mutable dump = None
+  val mutable relaying = false
+  val mutable ns = []
+  val mutable decoder = fun _ -> raise NoDecoder
+  val mutable stype = None
 
-   method login : (string option)*(string -> string -> bool) = login
+  val mutable dump = None
+  val mutable logf = None
 
-   method stype = Source.Fallible
+  method login : (string option)*(string -> string -> bool) = login
 
-   (* Insert metadata *)
-   method insert_metadata m =
+  method stype = Source.Fallible
+
+  (* Insert metadata *)
+  method insert_metadata m =
     self#log#f 3 "New metadata chunk \"%s -- %s\""
-                (try Hashtbl.find m "artist" with _ -> "?")
-                (try Hashtbl.find m "title" with _ -> "?") ;
-    let n = Fmt.ticks_of_samples (Http_source.Generator.length abg) in
-    metadata <- (n,m) :: metadata
+      (try Hashtbl.find m "artist" with _ -> "?")
+      (try Hashtbl.find m "title" with _ -> "?") ;
+    Generator.add_metadata abg m
 
-   method put sample_freq data =
+  method put sample_freq data =
     if not relaying then failwith "relaying stopped" ;
     Mutex.lock lock ;
     (* TODO There must be two ways of handling overfull generator:
@@ -71,14 +75,14 @@ object (self)
         Mutex.lock lock ;
         if Generator.length abg >= abg_max_len then
         (* Here, we drop some data after the maximun buffer has been filled.
-         * Delaying the function can lead to deconnection/connection cycles 
+         * Delaying the function can lead to deconnection/connection cycles
          * when the source is not pulled. *)
-          Generator.remove abg (Generator.length abg - abg_max_len) 
+          Generator.remove abg (Generator.length abg - abg_max_len)
       end ;
     Generator.feed abg ~sample_freq data ;
     Mutex.unlock lock
 
-  method register_decoder s = 
+  method register_decoder s =
     match
       Http_source.stream_decoders#get s
     with
@@ -87,18 +91,39 @@ object (self)
 
   method get_type = stype
 
-  method feed socket  = 
+  method feed socket  =
     self#log#f 3 "Decoding..." ;
-    let close () = () in 
-    let read len = 
+    let close () = () in
+    let t0 = Unix.gettimeofday () in
+    let read len =
       let buf = String.make len ' ' in
+      let () =
+        let rec wait n =
+          let l,_,_ = Unix.select [socket] [] [] 1. in
+            if l=[] then begin
+              self#log#f 4 "No network activity for %d second(s)." n ;
+              if float n >= Harbor.conf_timeout#get then
+               begin
+                self#log#f 4 "network activity timeout! disconnecting source" ;
+                self#disconnect
+               end
+              else
+               wait (n+1)
+            end
+        in wait 1
+      in
       let input = Unix.read socket buf 0 len in
       if input<=0 then raise End_of_file ;
       let s = String.sub buf 0 input in
-      begin
-        match dump with 
-          | Some b -> output_string b s
-          | None -> ()
+      begin match dump with
+        | Some b -> output_string b s
+        | None -> ()
+      end ;
+      begin match logf with
+        | Some b ->
+            let time = (Unix.gettimeofday () -. t0) /. 60. in
+              Printf.fprintf b "%f %d\n%!" time self#length
+        | None -> ()
       end ;
       s
     in
@@ -106,47 +131,76 @@ object (self)
       { put = self#put ; read = read ;
         insert_metadata = self#insert_metadata ; close = close }
     in
+     begin
       try decoder sink with
         | e -> self#log#f 2 "Feeding stopped: %s" (Printexc.to_string e) ;
+               if debug then raise e
+     end;
     self#disconnect ;
     try
-      Unix.shutdown socket Unix.SHUTDOWN_ALL ;
       Unix.close socket
     with
       | _ -> ()
 
-  method wake_up _ = 
+  method private wake_up _ =
     if ns = [] then
       ns <- Server.register [self#id] "input.harbor" ;
     self#set_id (Server.to_string ns) ;
-    let stop _ = 
+    let stop _ =
       if relaying then (self#disconnect ; "Done")
       else "No source client connected"
     in
-    Server.add ~ns "stop" ~descr:"Stop current source client, if connected." stop ;
-    Server.add ~ns "kick" ~descr:"Kick current source client, if connected." stop ;
-    Server.add ~ns "status" ~descr:"Display current status."
-      (fun _ -> if relaying then "source client connected" else "no source client connected")
+    Server.add
+      ~ns "stop" ~descr:"Stop current source client, if connected." stop ;
+    Server.add
+      ~ns "kick" ~descr:"Kick current source client, if connected." stop ;
+    Server.add
+      ~ns "status" ~descr:"Display current status."
+      (fun _ ->
+         if relaying then
+           "source client connected"
+         else
+           "no source client connected")
 
-  method relay socket = 
+  method private sleep =
+    if relaying then self#disconnect
+
+  method relay (headers:(string*string) list) socket =
     relaying <- true ;
-    on_connect () ;
-    begin
-      match dumpfile with
-        | Some f -> dump <- Some (open_out_bin f)
-        | None -> ()
+    let headers = List.map (fun (x,y) -> String.lowercase x,y) headers in
+    on_connect headers ;
+    begin match dumpfile with
+      | Some f ->
+          begin try
+            dump <- Some (open_out_bin (Utils.home_unrelate f))
+          with e ->
+            self#log#f 2 "Could not open dump file: %s" (Printexc.to_string e)
+          end
+      | None -> ()
+    end ;
+    begin match logfile with
+      | Some f ->
+          begin try
+            logf <- Some (open_out_bin (Utils.home_unrelate f))
+          with e ->
+            self#log#f 2 "Could not open log file: %s" (Printexc.to_string e)
+          end
+      | None -> ()
     end ;
     ignore (Tutils.create
               (fun () -> self#feed socket) ()
               "harbor source feeding")
 
-  method disconnect = 
+  method disconnect =
     if relaying then on_disconnect () ;
-    begin
-      match dump with
-        | Some f -> close_out f
-        | None -> ()
-    end ;    
+    begin match dump with
+      | Some f -> close_out f ; dump <- None
+      | None -> ()
+    end ;
+    begin match logf with
+      | Some f -> close_out f ; logf <- None
+      | None -> ()
+    end ;
     relaying <- false
 
   method is_taken = relaying
@@ -164,9 +218,13 @@ let () =
         "max", Lang.float_t, Some (Lang.float 10.),
         Some "Maximum duration of the buffered data.";
 
-        "on_connect", Lang.fun_t [] Lang.unit_t,
-        Some (Lang.val_cst_fun [] Lang.unit),
-        Some "Functions to excecute when a source is connected";
+        "on_connect", Lang.fun_t 
+        [false,"",Lang.list_t (Lang.product_t Lang.string_t Lang.string_t)] Lang.unit_t,
+        Some (Lang.val_cst_fun 
+          ["","headers",None] Lang.unit),
+        Some "Function to execute when a source is connected. \
+              Its receives the list of headers, of the form: \
+              (\"label\",\"value\"). All labels are lowercase.";
 
         "on_disconnect",Lang.fun_t [] Lang.unit_t,
         Some (Lang.val_cst_fun [] Lang.unit),
@@ -174,24 +232,34 @@ let () =
 
         "user",Lang.string_t,
         Some (Lang.string ""),
-        Some "Source user. Override default if not empty, except for icy protocol.";
+        Some "Source user. Override default if not empty, \
+              except for icy protocol.";
 
         "password",Lang.string_t,
         Some (Lang.string ""),
-        Some "Source password. Override default if not empty, except for icy protocol";
+        Some "Source password. Override default if not empty, \
+              except for icy protocol";
 
-        "auth",Lang.fun_t [false,"",Lang.string_t;false,"",Lang.string_t] 
+        "auth",Lang.fun_t [false,"",Lang.string_t;false,"",Lang.string_t]
                                     Lang.bool_t,
-        Some (Lang.val_cst_fun ["","login",None;"","password",None]
-	                   (Lang.bool false)),
+        Some (Lang.val_cst_fun
+                ["","login",None;"","password",None]
+                (Lang.bool false)),
         Some "Authentification function. \
-	      <code>f(login,password)</code> returns <code>true</code> \
-	      if the user should be granted access for this login. \
-	      Override any other method if used.";
+              <code>f(login,password)</code> returns <code>true</code> \
+              if the user should be granted access for this login. \
+              Override any other method if used.";
 
         "dumpfile", Lang.string_t, Some (Lang.string ""),
         Some "Dump stream to file, for debugging purpose. Disabled if empty.";
 
+        "logfile", Lang.string_t, Some (Lang.string ""),
+        Some "Log buffer status to file, for debugging purpose. \
+              Disabled if empty.";
+
+        "debug", Lang.bool_t, Some (Lang.bool false),
+        Some "Run in debugging mode by not catching some exceptions.";
+
         "", Lang.string_t, None,
         Some "Mountpoint to look for." ]
       (fun p ->
@@ -202,49 +270,67 @@ let () =
          in
          let trivially_false = function
            | { Lang.value =
-              Lang.Fun (_,_,_,{ Lang_values.term = Lang_values.Bool false }) } -> true
+                 Lang.Fun (_,_,_,
+                           { Lang_values.term = Lang_values.Bool false }) }
+               -> true
            | _ -> false
          in
          let user = Lang.to_string (List.assoc "user" p) in
          let password = Lang.to_string (List.assoc "password" p) in
-	 let auth_function = List.assoc "auth" p in
+         let debug = Lang.to_bool (List.assoc "debug" p) in
+         let auth_function = List.assoc "auth" p in
          let login user pass =
-           let user_login test_user test_pass = 
+           let user_login test_user test_pass =
              let user,pass =
                let f g x = match x with "" -> g | _ -> x in
                f Harbor.conf_harbor_user#get user,
                f Harbor.conf_harbor_pass#get password
-             in  
+             in
              test_user = user &&
              test_pass = pass
            in
-	   if not (trivially_false auth_function) then
-             Lang.to_bool 
-               (Lang.apply auth_function 
-                 ["",Lang.string user;
-                  "",Lang.string pass])
-            else user_login user pass
+             if not (trivially_false auth_function) then
+               Lang.to_bool
+                 (Lang.apply auth_function
+                    ["",Lang.string user;
+                     "",Lang.string pass])
+             else
+               user_login user pass
          in
-	 let login = 
+         let login =
            let f x = if x <> "" then Some x else None in
            (f user, login)
-         in	
-         let dumpfile = 
+         in
+         let dumpfile =
            match Lang.to_string (List.assoc "dumpfile" p) with
              | "" -> None
              | s -> Some s
          in
+         let logfile =
+           match Lang.to_string (List.assoc "logfile" p) with
+             | "" -> None
+             | s -> Some s
+         in
          let bufferize = Lang.to_float (List.assoc "buffer" p) in
          let max = Lang.to_float (List.assoc "max" p) in
-	 let on_connect = fun () -> ignore (Lang.apply 
-	                       (List.assoc "on_connect" p) []) in
-	 let on_disconnect = fun () -> ignore (Lang.apply
+         let on_connect l =
+           let l = 
+             List.map 
+              (fun (x,y) -> Lang.product (Lang.string x) (Lang.string y))
+              l
+           in
+           let arg = Lang.list l in
+           ignore (Lang.apply (List.assoc "on_connect" p) ["",arg])
+         in
+         let on_disconnect = fun () -> ignore (Lang.apply
                                (List.assoc "on_disconnect" p) []) in
            try
              ((Harbor.find_source mount):>Source.source)
            with
              | Not_found ->
                  Harbor.add_source mount
-                   ((new http_input_server ~bufferize ~max ~login ~dumpfile
-		               ~on_connect ~on_disconnect):>Harbor.source) ;
+                   ((new http_input_server
+                       ~bufferize ~max ~login
+                       ~dumpfile ~logfile
+                       ~on_connect ~on_disconnect ~debug):>Harbor.source) ;
                  ((Harbor.find_source mount):>Source.source))
