@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ let decode_aac sink =
       | e ->
           aac_stream_log#f 2 "Read error %s" (Printexc.to_string e)
   in
-  let samplerate, channels =
+  let (* ofs *) _, samplerate, channels = (* TODO: use ofs *)
     feed_buf ();
     let n =
       try
