@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -57,7 +57,6 @@ let stream_decoders : (sink -> unit) Plug.plug =
 
 (** Utilities for reading icy metadata *)
 
-let log = Dtools.Log.make ["readmeta"]
 let read_metadata () = let old_chunk = ref "" in fun socket ->
   let size =
     let buf = " " in
@@ -93,13 +92,11 @@ let read_metadata () = let old_chunk = ref "" in fun socket ->
         parse (String.sub s (close+1) ((String.length s)-close-1))
     with _ -> ()
   in
-    if chunk = "" then begin
-      (* log#f 4 "Empty chunk!" ; *)
+    if chunk = "" then
       None
-    end else if chunk = !old_chunk then begin
-      (* log#f 4 "Redundant chunk (%S)!" chunk ; *)
+    else if chunk = !old_chunk then
       None
-    end else begin
+    else begin
       old_chunk := chunk ;
       parse chunk ;
       Some h
@@ -199,9 +196,13 @@ module Generated = Generated.From_Float_pcm_Generator
 (* Used to handle redirections. *)
 exception Redirection of string
 
-class http ~playlist_mode ~poll_delay ~timeout
-           ~bind_address ~autostart ~bufferize ~max url =
-  let abg_max_len = Fmt.samples_of_seconds max in
+class http ~playlist_mode ~poll_delay ~timeout ~track_on_meta ?(force_mime=None)
+           ~bind_address ~autostart ~bufferize ~max
+           ~debug ?(logfile=None)
+           ~user_agent url =
+  let abg_max_len =
+    Fmt.samples_of_seconds (Pervasives.max max bufferize)
+  in
 object (self)
   inherit Source.source
   inherit Generated.source
@@ -219,41 +220,36 @@ object (self)
   val polling_lock = Mutex.create ()
   val polling_cond = Condition.create ()
 
+  val mutable logf = None
+
   val mutable connected = false
   val mutable relaying = autostart
   val mutable playlist_mode = playlist_mode
 
-
   (* Insert metadata *)
   method insert_metadata m =
     self#log#f 3 "New metadata chunk \"%s -- %s\""
                 (try Hashtbl.find m "artist" with _ -> "?")
                 (try Hashtbl.find m "title" with _ -> "?") ;
-    metadata <- (Fmt.ticks_of_samples (Generator.length abg), m) :: metadata ;
+    Generator.add_metadata abg m ;
+    if track_on_meta then Generator.add_break abg ;
 
   (* Feed the buffer generator *)
-
   method put sample_freq data =
     if not relaying then failwith "relaying stopped" ;
+    if poll_should_stop then failwith "source stopped" ;
     Mutex.lock lock ;
-    (* TODO There must be two ways of handling overfull generator:
-     * (1) when streaming, one should just stop the decoder for a while;
-     * (2) when not streaming, one should throw some data.
-     * Doing 1 instead of 2 can lead to deconnections.
-     * Doing 2 instead of 1 leads to ugly sound.
-     * The problem is:
-     * we don't know when the source will really be asked for data... *)
-    if Generator.length abg >= abg_max_len then
-      begin
-        Mutex.unlock lock ;
-        Thread.delay (max /. 3.) ;
-        Mutex.lock lock ;
-        if Generator.length abg >= abg_max_len then
-        (* Here, we drop some data after the maximun buffer has been filled.
-         * Delaying the function can lead to deconnection/connection cycles
-         * when the source is not pulled. *)
-          Generator.remove abg (Generator.length abg - abg_max_len)
-      end ;
+    (* Drop data when buffer is full. This is the only way
+     * to make it work with switches, when input.http is not
+     * pulled for some time.
+     * It can also be necessary if liquidsoap is lower than the stream. *)
+    if Generator.length abg >= abg_max_len then begin
+      if Generator.length abg >= abg_max_len then begin
+        (* TODO maybe insert a break in the generator *)
+        self#log#f 6 "Overfull buffer: dropping data." ;
+        Generator.remove abg (Generator.length abg - abg_max_len)
+      end
+    end;
     Generator.feed abg ~sample_freq data ;
     Mutex.unlock lock
 
@@ -261,18 +257,52 @@ object (self)
     connected <- true ;
     let close () = Http.disconnect socket in
     let read = read_stream socket chunked metaint self#insert_metadata in
+    let read =
+      match logf with
+        | None -> read
+        | Some f ->
+            let t0 = Unix.gettimeofday () in
+              fun len ->
+                let ret : string = read len in
+                let time = (Unix.gettimeofday () -. t0) /. 60. in
+                  Printf.fprintf f "%f %d\n%!" time self#length ;
+                  ret
+    in
     let sink =
       { put = self#put ; read = read ;
         insert_metadata = self#insert_metadata ; close = close }
     in
-      try dec sink with
-        | e -> self#log#f 2 "Feeding stopped: %s" (Printexc.to_string e)
+      try
+       (* Starting decoding: adding a break here *)
+       Generator.add_break abg ;
+       dec sink
+      with
+        | e ->
+            (* Feeding has stopped: adding a break here. *)
+            Generator.add_break abg ;
+            begin match e with
+              | Failure s ->
+                  self#log#f 2 "Feeding stopped: %s." s
+              | e ->
+                  self#log#f 2 "Feeding stopped: %s" (Printexc.to_string e)
+            end ;
+            begin match logf with
+              | Some f -> close_out f ; logf <- None
+              | None -> ()
+            end ;
+            if debug then raise e
 
-  method connect = self#private_connect
+  method connect = self#private_connect ~encode:true
 
   (* Called when there's no decoding process, in order to create one. *)
-  method private_connect url =
-    let host,port,mount,auth = parse_url (Http.http_encode url) in
+  method private_connect ?(encode=true) url =
+    let url =
+      if encode then
+        Http.http_encode url
+      else
+        url
+    in
+    let host,port,mount,auth = parse_url url in
     let req =
       Printf.sprintf
         "GET %s HTTP/1.0\r\nHost: %s:%d\r\n"
@@ -285,8 +315,8 @@ object (self)
     in
     let request =
       Printf.sprintf
-        "%sUser-Agent: liquidsoap/%s (%s; ocaml %s)\r\n%sIcy-MetaData:1\r\n\r\n"
-        req Configure.version Sys.os_type Sys.ocaml_version auth
+        "%sUser-Agent: %s\r\n%sIcy-MetaData:1\r\n\r\n"
+        req user_agent auth
     in
       self#log#f 4 "Connecting to <http://%s:%d%s>..." host port mount ;
       try
@@ -296,14 +326,17 @@ object (self)
           try
             let (_, status, status_msg), fields = Http.request socket request in
             let content_type =
-              try List.assoc "content-type" fields with Not_found -> "unknown"
-            in
-            (* Remove modifiers from content type. *)
-            let content_type =
-              try
-                let sub = Pcre.exec ~pat:"^([^;]+);.*$" content_type in
-                  Pcre.get_substring sub 1
-              with
+              match force_mime with
+                | Some s -> s
+                | None ->
+              let content_type =
+                try List.assoc "content-type" fields with Not_found -> "unknown"
+              in
+              (* Remove modifiers from content type. *)
+                try
+                  let sub = Pcre.exec ~pat:"^([^;]+);.*$" content_type in
+                    Pcre.get_substring sub 1
+                with
                 | Not_found -> content_type
             in
             let metaint =
@@ -389,6 +422,18 @@ object (self)
                           | Some d -> d
                           | None -> failwith "Unknown format!"
                       in
+                        begin match logfile with
+                          | Some f ->
+                              begin try
+                                logf <-
+                                Some (open_out_bin (Utils.home_unrelate f))
+                              with e ->
+                                self#log#f 2
+                                  "Could not open log file: %s"
+                                  (Printexc.to_string e)
+                              end
+                          | None -> ()
+                        end ;
                         self#log#f 3 "Decoding..." ;
                         self#feeding dec socket chunked metaint
           with
@@ -397,7 +442,7 @@ object (self)
                 raise e
       with
         | Redirection location ->
-            self#private_connect location
+            self#private_connect ~encode:false location
         | Http.Error e ->
             self#log#f 4 "Connection failed: %s!" (Http.string_of_error e)
         | e ->
@@ -425,7 +470,7 @@ object (self)
     if ns = [] then
       ns <- Server.register [self#id] "input.http" ;
     self#set_id (Server.to_string ns) ;
-    Server.add ~ns "start" ~usage:"start" ~descr:"Start the source, if needed." 
+    Server.add ~ns "start" ~usage:"start" ~descr:"Start the source, if needed."
        (fun _ -> relaying <- true ; "Done") ;
     Server.add ~ns "stop" ~usage:"stop" ~descr:"Stop the source if streaming."
        (fun _ -> relaying <- false ; "Done")
@@ -454,6 +499,12 @@ let () =
         "timeout", Lang.float_t, Some (Lang.float 10.),
         Some "Timeout for http connection." ;
 
+        "new_track_on_metadata", Lang.bool_t, Some (Lang.bool true),
+        Some "Treat new metadata as new track." ;
+
+        "force_mime", Lang.string_t, Some (Lang.string ""),
+        Some "Force mime data type. Not used if empty." ;
+
         "playlist_mode", Lang.string_t, Some (Lang.string "normal"),
         Some "Valid modes are \"normal\", \"random\", \"randomize\" \
               and \"first\". The first ones have the same meaning as for \
@@ -461,39 +512,69 @@ let () =
               discards all entries but the first one." ;
 
         "poll_delay", Lang.float_t, Some (Lang.float 2.),
-        Some "Polling delay." ;
+        Some "Polling delay when trying to connect to the stream." ;
 
         "max", Lang.float_t, Some (Lang.float 10.),
         Some "Maximum duration of the buffered data." ;
 
+        "logfile", Lang.string_t, Some (Lang.string ""),
+        Some "Log buffer status to file, for debugging purpose. \
+              Disabled if empty." ;
+
+        "debug", Lang.bool_t, Some (Lang.bool false),
+        Some "Run in debugging mode, not catching some exceptions." ;
+
+        "user_agent", Lang.string_t,
+        Some (Lang.string
+            (Printf.sprintf "liquidsoap/%s (%s; ocaml %s)"
+                Configure.version Sys.os_type Sys.ocaml_version)),
+        Some "User agent." ;
+
         "", Lang.string_t, None,
         Some "URL of an http stream (default port is 80)." ]
       (fun p ->
-           let playlist_mode =
-             let s = List.assoc "playlist_mode" p in
-           match Lang.to_string s with
-             | "random" -> Random
-             | "first" -> First
-             | "randomize" -> Randomize
-             | "normal" -> Normal
-             | _ ->
-          raise
-            (Lang.Invalid_value
-               (s,
-                "valid values are 'random', 'randomize', 'normal' and 'first'"))
+         let playlist_mode =
+           let s = List.assoc "playlist_mode" p in
+             match Lang.to_string s with
+               | "random" -> Random
+               | "first" -> First
+               | "randomize" -> Randomize
+               | "normal" -> Normal
+               | _ ->
+                   raise
+                     (Lang.Invalid_value
+                        (s,
+                         "valid values are 'random', 'randomize', \
+                          'normal' and 'first'"))
          in
          let url = Lang.to_string (List.assoc "" p) in
          let autostart = Lang.to_bool (List.assoc "autostart" p) in
          let bind_address = Lang.to_string (List.assoc "bind_address" p) in
+         let user_agent = Lang.to_string (List.assoc "user_agent" p) in
+         let track_on_meta =
+           Lang.to_bool (List.assoc "new_track_on_metadata" p)
+         in
+         let debug = Lang.to_bool (List.assoc "debug" p) in
+         let logfile =
+           match Lang.to_string (List.assoc "logfile" p) with
+             | "" -> None
+             | s -> Some s
+         in
          let bind_address =
            match bind_address with
              | "" -> None
              | s -> Some s
          in
+         let force_mime =
+           match Lang.to_string (List.assoc "force_mime" p) with
+             | "" -> None
+             | s  -> Some s
+         in
          let bufferize = Lang.to_float (List.assoc "buffer" p) in
          let timeout = Lang.to_float (List.assoc "timeout" p) in
          let max = Lang.to_float (List.assoc "max" p) in
          let poll_delay = Lang.to_float (List.assoc "poll_delay" p) in
-           ((new http ~playlist_mode ~timeout ~autostart
-                      ~bind_address ~poll_delay ~bufferize ~max url)
-              :>Source.source))
+           ((new http ~playlist_mode ~timeout ~autostart ~track_on_meta
+                      ~force_mime ~bind_address ~poll_delay
+                      ~bufferize ~max ~debug ~logfile ~user_agent url)
+              :> Source.source))
