@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,10 +22,15 @@
 
 open Lastfm
 
-class lastfm ~autostart ~poll_delay ~submit ~bufferize ~timeout ~bind_address ~max uri =
+class lastfm ~autostart ~poll_delay ~submit ~track_on_meta 
+             ~bufferize ~timeout ~bind_address 
+             ~debug ~max ~user_agent uri =
 let playlist_mode = Http_source.First in
 object (self)
-  inherit Http_source.http ~playlist_mode ~poll_delay ~timeout ~autostart ~bind_address ~bufferize ~max uri as http
+  inherit Http_source.http ~playlist_mode ~poll_delay ~timeout 
+                           ~autostart ~bind_address ~bufferize 
+                           ~max ~track_on_meta 
+                           ~debug ~user_agent uri as http
 
   val mutable session = None
 
@@ -65,7 +70,7 @@ object (self)
         if submit then 
           begin
             match login with
-             | Some s -> Liqfm.submit (s.user,s.password) Liqfm.Lastfm [metas]
+             | Some s -> Liqfm.submit (s.user,s.password) true Liqfm.Lastfm [metas]
              | None -> 
                  self#log#f 3 
                    "Lastfm Submission failed: not authenficated"
@@ -112,8 +117,17 @@ let () =
         Some "Submit song to Audioscrobbler. \
            Only when the url is not anonymous, e.g. \
            <code>lastfm://user:password@artist/foo</code>." ;
+        "new_track_on_metadata", Lang.bool_t, Some (Lang.bool true),
+        Some "Treat new metadata as new track." ;
+        "debug", Lang.bool_t, Some (Lang.bool false),
+        Some "Run in debugging mode by not catching some exceptions." ;
         "max", Lang.float_t, Some (Lang.float 10.),
         Some "Maximum duration of the buffered data." ;
+       "user_agent", Lang.string_t,
+        Some (Lang.string
+            (Printf.sprintf "liquidsoap/%s (%s; ocaml %s)"
+                Configure.version Sys.os_type Sys.ocaml_version)),
+        Some "User agent." ;
         "", Lang.string_t, None,
         Some "URI of a lastfm  stream (e.g. lastfm://user/toots5446/playlist)."
       ]
@@ -121,14 +135,19 @@ let () =
          let uri = Lang.to_string (List.assoc "" p) in
          let autostart = Lang.to_bool (List.assoc "autostart" p) in
          let submit = Lang.to_bool (List.assoc "submit" p) in
+         let track_on_meta = Lang.to_bool (List.assoc "new_track_on_metadata" p) in
+         let debug = Lang.to_bool (List.assoc "debug" p) in
          let bind_address = Lang.to_string (List.assoc "bind_address" p) in
          let bind_address =
            match bind_address with
              | "" -> None
              | s -> Some s
          in
+         let user_agent = Lang.to_string (List.assoc "user_agent" p) in
          let bufferize = Lang.to_float (List.assoc "buffer" p) in
 	 let timeout = Lang.to_float (List.assoc "timeout" p) in
          let poll_delay =  Lang.to_float (List.assoc "poll_delay" p) in
          let max = Lang.to_float (List.assoc "max" p) in
-           ((new lastfm ~autostart ~submit ~poll_delay ~bufferize ~bind_address ~timeout ~max uri):>Source.source))
+           ((new lastfm ~autostart ~submit ~poll_delay ~bufferize ~track_on_meta 
+                        ~bind_address ~timeout ~max ~debug 
+                        ~user_agent uri):>Source.source))
