@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,17 +27,19 @@ type float_pcm_t = float (* samplerate *)
 type track_t =
   | Float_pcm_t of float_pcm_t
   | Midi_t of Midi.header
+  | RGB_t
 
 type track =
   | Float_pcm of (float_pcm_t * float_pcm)
   | Midi of (Midi.header * Midi.track ref)
+  | RGB of RGB.t array
 
 type metadata = (string,string) Hashtbl.t
 
 type t =
     {
       freq : int; (* ticks per second *)
-      length : int;
+      length : int; (* length in ticks *)
       mutable tracks : track array;
       (* End of track markers.
        * A break at the end of the buffer is not an end of track.
@@ -62,6 +64,10 @@ let create_track freq length = function
         Float_pcm (f, Array.create len 0.)
   | Midi_t h ->
       Midi (h, ref (Midi.create_track ()))
+  | RGB_t ->
+      RGB (Array.init
+             (Fmt.video_frames_per_frame ())
+             (fun _ -> RGB.create (Fmt.video_width ()) (Fmt.video_height ())))
 
 let create kind ~freq ~length =
   {
@@ -72,9 +78,27 @@ let create kind ~freq ~length =
     metadata = [];
   }
 
+(* Create a standard frame. *)
+let make () =
+  let audio =
+    Array.init (Fmt.channels ())
+      (fun _ ->
+         Float_pcm_t (float (Fmt.samples_per_second()))
+      )
+  in
+  let video =
+    Array.init (Fmt.video_channels ()) (fun _ -> RGB_t)
+  in
+    create
+      (Array.append audio video)
+      (Fmt.ticks_per_second ())
+      (Fmt.ticks_per_frame ())
+
+
 let kind_of_track = function
   | Float_pcm (t, _) -> Float_pcm_t t
   | Midi (h, _) -> Midi_t h
+  | RGB _ -> RGB_t
 
 let kind b = Array.map kind_of_track b.tracks
 
@@ -127,6 +151,8 @@ let free_metadata b t =
 let free_all_metadata b = b.metadata <- []
 let get_all_metadata b = List.filter (fun (x,_) -> x <> -1) b.metadata
 let set_all_metadata b l = b.metadata <- l
+let get_past_metadata b =
+  try Some (List.assoc (-1) b.metadata) with Not_found -> None
 
 
 (** Chunks *)
@@ -146,10 +172,15 @@ let blit src src_pos dst dst_pos len =
           let c x = int_of_float (float x *. r) in
             float_blit a (c src_pos) a' (c dst_pos) (c len)
       | Midi (_,m), Midi (h,m') -> m' := !m
+      | RGB src, RGB dst ->
+          (* TODO: handle offsets! *)
+          for i = 0 to Array.length src - 1 do
+            RGB.blit_fast src.(i) dst.(i)
+          done
       | _, _ -> assert false
   done
 
-let log = Dtools.Log.make ["prout"]
+let log = Dtools.Log.make ["frame"]
 
 (* Get the (end of) next chunk from [from].
  * A chunk is a region of a frame between two breaks.
@@ -158,7 +189,6 @@ let get_chunk ab from =
   assert (is_partial ab);
   let p = position ab in
   let copy_chunk i =
-    if false then log#f 2 "%d-%d/%d" p i (size ab) ;
     add_break ab i ;
     blit from p ab p (i-p) ;
     (* If the last metadata before [p] differ in [from] and [ab],
@@ -169,22 +199,13 @@ let get_chunk ab from =
       let before_p l =
         match
           List.sort
-            (fun (a,_) (b,_) -> compare b a)      (* the greatest *)
+            (fun (a,_) (b,_) -> compare b a)     (* the greatest *)
             (List.filter (fun x -> fst x < p) l) (* that is less than p *)
         with [] -> None | x::_ -> Some (snd x)
       in
         match before_p from.metadata, before_p ab.metadata with
           | Some b, a ->
-              if a <> Some b then begin
-                if false then begin
-                  log#f 2 "XXXX %s" (Hashtbl.find b "title") ;
-                  match a with
-                    |  Some a ->
-                        if false then log#f 2 "YYYY %s" (Hashtbl.find a "title")
-                    | None -> ()
-                end ;
-                set_metadata ab p b
-              end
+              if a <> Some b then set_metadata ab p b
           | None, _ -> ()
     end ;
     (* Copy new metadata blocks for this chunk.
@@ -215,3 +236,57 @@ let get_chunk ab from =
             aux i tl
   in
     aux 0 (List.rev from.breaks)
+
+let fill_from_marshal stream frame =
+    assert(is_partial frame);
+    let cur_pos = position frame in
+    (* Yes, this might lose some data.
+     * However, it is very simple this way,
+     * and avoid either a local buffer or a
+     * send->receive paradigm with the other end.. *)
+    let rec get () =
+      let (nframe : t) = Marshal.from_channel stream in
+      if position nframe < cur_pos then
+        get ()
+      else
+        nframe
+    in
+    let nframe = get () in
+    let new_pos = position nframe in
+    let len = new_pos - cur_pos in
+    blit nframe cur_pos frame cur_pos len;
+    let new_meta = get_all_metadata nframe in
+    let cur_meta = get_all_metadata frame in
+    let add_meta (p,m) =
+      match p with
+        (* Last kept metadata should always be the more recent as possible.. *)
+        | -1 ->
+             if not (List.mem_assoc (-1)
+                      (get_all_metadata frame))
+             then
+               set_metadata frame (-1) m
+        | p when p >= cur_pos ->
+             set_metadata frame p m
+        (** The perfectionist's addition:
+          * Add a metadata in the worst case.. *)
+        | p when
+              (* No old metadata *)
+              cur_meta = [] &&
+              (* No new metadata, or kept metadata *)
+              not (List.exists
+                     (fun (x,_) -> (x >= cur_pos) || (x = -1))
+                     new_meta) &&
+              (* No metadata was already added at cur_pos.. *)
+              not (List.mem_assoc cur_pos
+                    (get_all_metadata frame))
+                        ->
+             (* Add this metadata at cur_pos. Since
+              * the list starts with the oldest one,
+              * this should always add the latest one,
+              * though I doubt another situation will
+              * ever happen.. *)
+             set_metadata frame cur_pos m
+        | _ -> ()
+    in
+    List.iter add_meta new_meta;
+    add_break frame new_pos
