@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,12 +41,13 @@ let conf_harbor_pass =
     "Default password for source connection."
 let conf_icy =
   Conf.bool ~p:(conf_harbor#plug "icy") ~d:false
-      "Enable the builtin ICY (shout) stream receiver. \
-       Custom password, user or authentification function don't apply \
-       to ICY mountpoints."
+   "Enable the builtin ICY (shout) stream receiver."
 let conf_timeout =
   Conf.float ~p:(conf_harbor#plug "timeout") ~d:30.
         "Timeout for source connections."
+let conf_pass_verbose =
+  Conf.bool ~p:(conf_harbor_pass#plug "verbose") ~d:false
+        "Display passwords, for debugging."
 
 let log = Log.make ["harbor"]
 
@@ -57,7 +58,7 @@ class virtual source =
 object(self)
   inherit Source.source
 
-  method virtual relay : Unix.file_descr -> unit
+  method virtual relay : (string*string) list -> Unix.file_descr -> unit
   method virtual insert_metadata : (string, string) Hashtbl.t -> unit
   method virtual login : (string option)*(string -> string -> bool)
   method virtual is_taken : bool
@@ -116,9 +117,22 @@ let http_error_page code status msg =
      <body><p>" ^ msg ^ "</p></body></html>\n" )
 
 let parse_icy_request_line r =
-      (match r with
-         | s when s = conf_harbor_pass#get -> Shout
-         | s -> Invalid(s) ),
+      (try
+        let s = find_source "/" in
+        let user,auth_f = s#login in
+        let user = 
+          match user with
+            | Some v -> v
+            | None -> conf_harbor_user#get
+        in
+        if auth_f user r then
+          Shout
+        else
+          Invalid(r)
+      with
+        | _ -> 
+          log#f 2 "No source registered for ICY mountpoint (\"/\").";
+          failwith "No ICY mountpoint."),
       "/",
       Icy
 
@@ -156,7 +170,10 @@ let parse_headers headers =
       | Not_found -> l
   in
   let headers = List.fold_right split_header headers [] in
-  List.iter (fun (h, v) -> log#f 4 "Header: %s, value: %s." h v) headers ;
+  let display_headers = 
+    List.filter (fun (x,_) -> conf_pass_verbose#get || x <> "AUTHORIZATION") headers
+  in
+  List.iter (fun (h, v) -> log#f 4 "Header: %s, value: %s." h v) display_headers ;
   headers
 
 let auth_check ~login c uri headers =
@@ -183,7 +200,8 @@ let auth_check ~login c uri headers =
           Str.split (Str.regexp ":") (Utils.decode64 (List.nth data 1))
         in
         let user,pass = List.nth auth_data 0, List.nth auth_data 1 in
-          log#f 4 "Requested username: %s, password: %s." user pass ;
+          if conf_pass_verbose#get then
+            log#f 4 "Requested username: %s, password: %s." user pass ;
           if not (auth_f user pass) then
             raise Not_authenticated ;
           (* OK *)
@@ -245,7 +263,7 @@ let handle_source_request ~icy hprotocol c uri headers =
           s#register_decoder stype ;
           log#f 3 "Adding source on mountpoint '%s' with type '%s'." uri stype ;
           if not icy then write_answer ~keep:true c "HTTP/1.0 200 OK\r\n\r\n" ;
-          s#relay c
+          s#relay headers c
   with
     | Mount_taken ->
         log#f 3 "Returned 401: Mount taken" ;
@@ -400,13 +418,14 @@ let handle_client ~icy socket =
                  | false -> parse_http_request_line
   in
   let process l =
-    let l =
-      match List.rev l with
-        | []
-        | _ :: [] -> assert false (* Should not happen *)
-        | e :: l -> List.rev l
-    in
     try
+      let l =
+       match List.rev l with
+         | []
+         | _ :: [] -> (* Should not happen *)
+             raise (Failure "Invalid input data")
+         | e :: l -> List.rev l
+      in
       let s =
         match l with
           | s :: _ -> s
@@ -421,7 +440,7 @@ let handle_client ~icy socket =
           | Get when not icy ->
               handle_get_request socket huri headers
           | Shout when icy ->
-              write_answer ~keep:true socket "OK\r\n\r\n" ;
+              write_answer ~keep:true socket "OK2\r\nicy-caps:11\r\n\r\n" ;
               handle_source_request ~icy:true hprotocol socket huri headers
           | Invalid s ->
               let er = if icy then "ICY " else "" in
@@ -435,7 +454,13 @@ let handle_client ~icy socket =
                  "The server did not understand your request.") ;
             failwith "cannot handle this, exiting"
     with
-      | Failure s -> log#f 3 "Failed: %s" s
+      | Failure s -> 
+          log#f 3 "Failed: %s" s;
+          try
+            Unix.shutdown socket Unix.SHUTDOWN_ALL ;
+            Unix.close socket
+          with
+            | _ -> ()
     in
       Duppy.Io.read ~priority ~recursive ~on_error
         Tutils.scheduler socket marker process
@@ -446,6 +471,7 @@ let start_harbor () =
   let rec incoming ~icy sock _ =
     begin
       try
+        if !Root.shutdown then failwith "shutting down" ;
         let (socket,caller) = accept sock in
         let ip =
           let a = match caller with
@@ -462,9 +488,13 @@ let start_harbor () =
         Unix.setsockopt_float socket Unix.SO_SNDTIMEO conf_timeout#get ;
         handle_client icy socket ;
         log#f 3 "New client: %s" ip
-      with e -> log#f 2 "Failed to accept new client: %S" (Printexc.to_string e)
+      with e -> log#f 2 "Failed to accept new client: %s" (Printexc.to_string e)
     end ;
-    [{ Duppy.Task.
+    if !Root.shutdown then begin
+      (try Unix.close sock with _ -> ()) ;
+      []
+    end else
+      [{ Duppy.Task.
          priority = priority ;
          events = [`Read sock] ;
          handler = (incoming ~icy sock) }]
