@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,6 @@
 
  *****************************************************************************)
 
-external set_tcp_nodelay : Unix.file_descr -> bool -> unit = "stub_set_tcp_nodelay"
+let set_tcp_nodelay fd v = Unix.setsockopt fd Unix.TCP_NODELAY v
 
 
