@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
  *****************************************************************************)
 
-type source = User | Lastfm
+type source = User | Lastfm | Broadcast | Recommendation | Unknown
 
 (** Performs a sumbission to audioscrobbler *)
-val submit : string*string -> source -> Frame.metadata list -> unit
+val submit : string*string -> bool -> source -> Frame.metadata list -> unit
